/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.protocol.server.Target;
import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.TargetType;
import org.apache.abdera.protocol.server.impl.SimpleTarget;
import org.apache.abdera.protocol.Resolver;
import org.apache.abdera.protocol.Request;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.ResourceNotFoundException;

import java.util.Map;
import java.util.HashMap;
import java.net.URLDecoder;
import java.io.UnsupportedEncodingException;

public class RegistryResolver implements Resolver<Target> {
    private Log log = LogFactory.getLog(RegistryResolver.class);

    public static final String RESOURCE = "/resource/";
    public static final String ATOM = "/atom/";

    public static final TargetType TAGS_TYPE = TargetType.get("tags", true);
    public static final TargetType LOGS_TYPE = TargetType.get("logs", true);
    public static final TargetType RATINGS_TYPE = TargetType.get("ratings", true);
    public static final TargetType RENAME_TYPE = TargetType.get("rename", true);
    public static final TargetType COMMENTS_TYPE = TargetType.get("comments", true);
    public static final TargetType TAGURL_TYPE = TargetType.get("tagURL", true);
    public static final TargetType ASSOCIATIONS_TYPE = TargetType.get("associations", true);
    public static final TargetType RESTORE_TYPE = TargetType.get("restore", true);
    public static final TargetType ASPECT_TYPE = TargetType.get("aspect", true);
    public static final TargetType VERSIONS_TYPE = TargetType.get("versions", true);

    // These custom types we need to handle methods that Abdera doesn't deal with adequately
    public static final TargetType DELETE_TYPE = TargetType.get("delete", true);
    public static final TargetType COLLECTION_CUSTOM_TYPE = TargetType.get("col-custom", true);

    private Registry registry;
    private String baseURI;

    public RegistryResolver(Registry registry, String baseURI) {
        this.registry = registry;
        this.baseURI = baseURI;
    }

    static Map<String, TargetType> types;
    static {
        types = new HashMap<String, TargetType>();
        types.put("tags", TAGS_TYPE);
        types.put("logs", LOGS_TYPE);
        types.put("ratings", RATINGS_TYPE);
        types.put("comments", COMMENTS_TYPE);
        types.put("rename", RENAME_TYPE);
        types.put("tagURL", TAGURL_TYPE);
        types.put("associations", ASSOCIATIONS_TYPE);
        types.put("restore", RESTORE_TYPE);
        types.put("versions", VERSIONS_TYPE);
    }

    public Target resolve(Request request) {
        RequestContext context = (RequestContext)request;
        String uri = context.getUri().toString();
        try {
            uri = URLDecoder.decode(uri, "utf-8");
        } catch (UnsupportedEncodingException e) {
            log.error(e);
            return null;
        }
        // URI will start with the baseURI, which we need to strip off.
        uri = uri.substring(baseURI.length());
        context.setAttribute("pathInfo", uri);
        String [] parts = splitPath(uri);
        boolean hasColon = false;
        TargetType type = null;
        if (parts.length > 1) {
            // Store the split URL for later
            context.setAttribute("splitPath", parts);
            String discriminator = parts[1];
            int idx = discriminator.indexOf("?");
            if (idx > -1) {
                discriminator = discriminator.substring(0, idx);
            }
            idx = discriminator.indexOf(":");
            if (idx > -1) {
                discriminator = discriminator.substring(0, idx);
                hasColon = true;
            }

            if (discriminator.startsWith("aspect")) {
                type = ASPECT_TYPE;
            } else {
                type = types.get(discriminator);
            }

            // If we have a discriminator that we don't understand, return a 404
            if (type == null) return null;

            // For the rest of this code, we'll want the "raw" resource URI
            if (!hasColon || !type.equals(COMMENTS_TYPE))
                uri = parts[0];
        }

        System.out.println(uri);
        boolean isMedia = false;
        if (uri.startsWith(RESOURCE)) {
            uri = uri.substring(RESOURCE.length() - 1);
            isMedia = true;
        } else if (uri.startsWith(ATOM)) {
            uri = uri.substring(ATOM.length() - 1);
        } else if (uri.startsWith("/tags") && (uri.length() == 5 || uri.charAt(5) == '/')) {
            return new SimpleTarget(TAGURL_TYPE, context);
        } else {
            return null;
        }

        Resource resource = null;
        final String method = context.getMethod();
        try {
            // TODO - should this be secureRegistry?
            resource = registry.get(uri);
        } catch (ResourceNotFoundException e) {
            // If this is a straight-ahead POST to a non-existent directory, create it?
            if (method.equals("POST") && parts.length == 1) {
                // Need to create it...
                try {
                    Collection c = registry.newCollection();
                    registry.put(uri, c);
                    resource = registry.get(uri);
                } catch (RegistryException e1) {
                    log.error(e1);
                    return null;
                }
            }
        } catch (RegistryException e) {
            return null; // return 404
        }

        context.setAttribute("MyResolver", this);
        if (type == null) {
            if (method.equals("DELETE")) {
                // Unfortunately, deletions aren't quite handled the way we want them
                // in AbstractEntityCollectionProvider, so for now we're using an
                // extensionRequest to get it done.
                type = DELETE_TYPE;
            } else {
                if (resource instanceof Collection) {
                    if (method.equals("HEAD") || method.equals("PUT")) {
                        // Abdera doesn't handle HEAD or PUT on collections yet... this should
                        // go away once that's fixed!
                        type = COLLECTION_CUSTOM_TYPE;
                    } else {
                        type = TargetType.TYPE_COLLECTION;
                    }
                } else {
                    type = isMedia ? TargetType.TYPE_MEDIA : TargetType.TYPE_ENTRY;
                }
            }
        }
        return new ResourceTarget(type, context, resource);
    }

    /**
     * This method will split the path into parts around the path separator, returning an array of
     * the parts.
     *
     * @param path : URL path
     * @return String array of the splited string
     */
    private String[] splitPath(String path) {
        return path.split("\\" + RegistryConstants.URL_SEPARATOR);
    }

    public String getBaseURI() {
        return baseURI;
    }
}
